package hello;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import hello.entity.Member;
import hello.repository.MemberRepository;
import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.WebIntegrationTest;
import org.springframework.http.MediaType;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import java.util.List;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static org.springframework.test.web.servlet.setup.MockMvcBuilders.standaloneSetup;

/**
 * Created by jackdaw on 16. 11. 10.
 */

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = Application.class)
@WebIntegrationTest
public class HelloControllerTest {

    private MockMvc mockMvc;
//    @Autowired
//    MemberRe controller;
    @Autowired
    MemberRepository repo;
    Logger logger = Logger.getLogger(this.getClass());

    @Before
    public void setUp() throws Exception {
        mockMvc = standaloneSetup(repo).build();
        repo.save(new Member("1",1));
        repo.save(new Member("2",2));
        repo.save(new Member("2",2));
    }

    private String jsonStringFromObject(Object object) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        return mapper.writeValueAsString(object);
    }

    @After
    public void tearDown() throws Exception {
        repo.deleteAll();
    }

    @Test
    public void index() throws Exception {

        String jsonString = jsonStringFromObject(repo.findAll());
        logger.info(jsonString);
        mockMvc.perform(get("/member")).andExpect(status().isOk()).andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
        .andExpect(content().string(equalTo(jsonString)));
    }

    @Test
    public void getMember() throws Exception {
        List<Member> list = repo.findAll();
        String jsonString = this.jsonStringFromObject(list.get(0));
        logger.info(jsonString);
        mockMvc.perform(get("/member/{id}", list.get(0).getId()))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
                .andExpect(content().string(equalTo(jsonString)));
    }

    @Test
    public void createMember() throws Exception {
        Member b = new Member("111",1111);
        String jsonString = this.jsonStringFromObject(b);

        MvcResult result = mockMvc.perform(post("/member")
                .contentType(MediaType.APPLICATION_JSON)
                .content(jsonString))
                .andExpect(status().isOk())
                .andReturn();

        logger.info(result.getResponse().getContentAsString());
    }

    @Test
    public void patchMember() throws Exception {
        List<Member> list = repo.findAll();
        String jsonString = this.jsonStringFromObject(list.get(0));
        logger.info(jsonString);
        list.get(0).setName("test");
        list.get(0).setAge(9999);
        jsonString = this.jsonStringFromObject(list.get(0));


        MvcResult result = mockMvc.perform(
                patch("/member/{id}",list.get(0).getId()).
                        contentType(MediaType.APPLICATION_JSON).
                        content(jsonString)).
                andExpect(status().isOk()).
                andReturn();
        logger.info(result.getResponse().getContentAsString());
    }

    @Test
    public void updateMember() throws Exception {
        List<Member> list = repo.findAll();
        String jsonString = this.jsonStringFromObject(list.get(0));
        logger.info(jsonString);
        list.get(0).setName("test");
        list.get(0).setAge(9999);
        jsonString = this.jsonStringFromObject(list.get(0));


        MvcResult result = mockMvc.perform(
                put("/member/{id}",list.get(0).getId()).
                        contentType(MediaType.APPLICATION_JSON).
                        content(jsonString)).
                andExpect(status().isOk()).
                andReturn();
        logger.info(result.getResponse().getContentAsString());
    }

    @Test
    public void deleteMember() throws Exception {
        List<Member> list = repo.findAll();

        long id = list.get(0).getId();
        String jsonString = this.jsonStringFromObject(list);

        list.remove(0);
        String compareString = this.jsonStringFromObject(list);

        mockMvc.perform(delete("/member/{id}", id)
                .content(jsonString))
                .andExpect(status().isOk())
                .andExpect(content().string(equalTo(compareString)));




    }

}