package hello.repository;

import hello.Application;
import hello.entity.Member;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import java.util.List;

import static org.junit.Assert.assertEquals;

/**
 * Created by jackdaw on 16. 11. 10.
 */

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = Application.class)
public class MemberRepositoryTest {
    @Autowired
    MemberRepository repo;

    @Before
    public void setup() {
        repo.deleteAll();
        repo.save(new Member("1",1));
        repo.save(new Member("2",2));
        repo.save(new Member("3",3));
        repo.save(new Member("4",4));
        repo.save(new Member("5",5));
    }

    @After
    public void down() {
        repo.deleteAll();
    }

    @Test
    public void create() {
        Member m = new Member("test",1);
        Member n = repo.save(m);

        assertEquals(m.getId(),n.getId());
    }

    @Test
    public void getAll() {
        assertEquals(repo.findAll().size(),5);
    }

    @Test
    public void delete() {
        List<Member> l = repo.findAll();
        Member m = l.get(0);
        repo.delete(m.getId());
        assertEquals(repo.findAll().size(),4);
    }

    @Test
    public void update() {
        List<Member> l = repo.findAll();
        Member m = l.get(0);
        m.setName("TTTTT");

        Member n = repo.save(m);
        assertEquals(m.getName(),n.getName());
    }

}